--- orig/planet/__init__.py
+++ mod/planet/__init__.py
@@ -121,12 +121,8 @@
         because we discard the numbers and just need them to be relatively
         consistent between each other.
         """
-        planet_filter_re = None
-        if self.filter:
-            planet_filter_re = re.compile(self.filter, re.I)
-        planet_exclude_re = None
-        if self.exclude:
-            planet_exclude_re = re.compile(self.exclude, re.I)
+
+        global_filter = PlanetFilter(self.filter, self.exclude)
             
         items = []
         seen_guids = {}
@@ -134,40 +130,13 @@
             for item in channel._items.values():
                 if hidden or not item.has_key("hidden"):
 
-                    channel_filter_re = None
-                    if channel.filter:
-                        channel_filter_re = re.compile(channel.filter,
-                                                       re.I)
-                    channel_exclude_re = None
-                    if channel.exclude:
-                        channel_exclude_re = re.compile(channel.exclude,
-                                                        re.I)
-                    if (planet_filter_re or planet_exclude_re \
-                        or channel_filter_re or channel_exclude_re):
-                        title = ""
-                        if item.has_key("title"):
-                            title = item.title
-                        content = item.get_content("content")
-
-                    if planet_filter_re:
-                        if not (planet_filter_re.search(title) \
-                                or planet_filter_re.search(content)):
-                            continue
-
-                    if planet_exclude_re:
-                        if (planet_exclude_re.search(title) \
-                            or planet_exclude_re.search(content)):
-                            continue
-
-                    if channel_filter_re:
-                        if not (channel_filter_re.search(title) \
-                                or channel_filter_re.search(content)):
-                            continue
-
-                    if channel_exclude_re:
-                        if (channel_exclude_re.search(title) \
-                            or channel_exclude_re.search(content)):
-                            continue
+                    channel_filter = PlanetFilter(channel.filter, channel.exclude)
+
+                    if (not global_filter.keep(item)):
+                        continue
+
+                    if (not channel_filter.keep(item)):
+                        continue
 
                     if not seen_guids.has_key(item.id):
                         seen_guids[item.id] = 1;
@@ -195,6 +164,65 @@
 
         return [ i[-1] for i in items ]
 
+class PlanetFilter:
+
+    def __init__(self, filter, exclude):
+        self.regexps = []
+
+        if filter:
+            for line in filter.split('\n'):
+                item = PlanetFilterItem();
+                item.positive = 1;
+                if (item.parse(line)):
+                    self.regexps.append(item)
+        if exclude:
+            for line in exclude.split('\n'):
+                item = PlanetFilterItem();
+                item.positive = 0;
+                if (item.parse(line)):
+                    self.regexps.append(item)
+
+    def keep(self, chanitem):
+        for regexp in self.regexps:
+            content = None
+            if (regexp.keyword == 'any'):
+                content = " ".join([chanitem.get_content("content"), chanitem.title])
+            elif (regexp.keyword == 'content'):
+                content = chanitem.get_content("content")
+            else:
+                content = chanitem[regexp.keyword];
+
+            matchobj = regexp.pattern.search(content);
+            if (regexp.positive and not matchobj):
+                #log.debug("skip because of %s =~ %s", regexp.keyword, regexp.rawpattern);
+                return 0
+            if ((not regexp.positive) and matchobj):
+                #log.debug("skip because of %s !~ %s", regexp.keyword, regexp.rawpattern);
+                return 0
+        return 1
+
+class PlanetFilterItem:
+
+    def __init__(self):
+        self.positive = 1
+        self.keyword = 'any'
+        self.pattern = None
+        self.rawpattern = None
+
+    FILTER_SYNTAX = re.compile(r'\s*(?P<keyword>[a-z]+)\s*=~\s*/(?P<regexp>.*?)/$')
+
+    def parse(self, line):
+        matchobj = self.FILTER_SYNTAX.match(line)
+        if matchobj:
+            self.keyword, self.rawpattern = matchobj.group('keyword', 'regexp')
+            #log.debug("matched syntax %s =~ %s", self.keyword, self.rawpattern);
+        else:
+            self.keyword = 'any'
+            self.rawpattern = line
+            #log.debug("matched old syntax %s =~ %s", self.keyword, self.rawpattern);
+        self.pattern = re.compile(self.rawpattern, re.I)
+        return 1;
+
 class Channel(cache.CachedInfo):
     """A list of news items.
 
